@@ -15,6 +15,7 @@
 ## 3. Function gauss.hermite adopted with only minor modifications
 ## from package statMod(?)
 
+utils::globalVariables(c("ths", "link", "threshold", "optRes", "Niter"))
 
 clmm <-
   function(formula, data, weights, start, subset, 
@@ -32,15 +33,14 @@ clmm <-
   if(missing(formula))  stop("Model needs a formula")
   if(missing(contrasts)) contrasts <- NULL
   ## set control parameters:
-  control <- do.call(clmm.control, c(control, list(...)))
+  control <- getCtrlArgs(control, list(...))
   nAGQ <- as.integer(round(nAGQ))
-
   ## Extract y, X, Zt, offset (off), weights (wts):
   frames <- clmm.model.frame(mc, contrasts)
   if(control$method == "model.frame") return(frames)
   ## Test rank deficiency and possibly drop some parameters:
   ## X is guarantied to have an intercept at this point.
-  frames$X <- drop.coef(frames$X)
+  frames$X <- drop.coef(frames$X, silent=FALSE)
   ## Compute the transpose of the Jacobian for the threshold function,
   ## tJac and the names of the threshold parameters, alpha.names:
   ths <- makeThresholds(frames$y, threshold)
@@ -67,7 +67,10 @@ clmm <-
   rho$par <- start
 
   ## Update rho with RE information:
-  if(ntau == 1L) {
+  ## only use clmm.fit.ssr if useMatrix=FALSE and there is a single
+  ## random effects term:
+  use.ssr <- (ntau == 1L && !control$useMatrix)
+  if(use.ssr) {
     rho.clm2clmm.ssr(rho=rho, grList=frames$grList,
                      ctrl=control$ctrl)
     set.AGQ(rho, nAGQ)
@@ -75,17 +78,21 @@ clmm <-
   else
     rho.clm2clmm(rho=rho, Zt=frames$Zt, grList=frames$grList,
                  ctrl=control$ctrl)
+  ## Stop if arguments are incompatible:
   if(nAGQ != 1 && ntau > 1)
-    stop("Quadrature methods are not available with
-more than one random effects term", call.=FALSE)
-  
+    stop(gettextf("Quadrature methods are not available with more than one random effects term"),
+         call.=FALSE) 
+  if(nAGQ != 1 && control$useMatrix)
+    stop(gettextf("Quadrature methods are not available with 'useMatrix = TRUE'"),
+         call.=FALSE) 
+    
   ## Possibly return the environment, rho without fitting:
   if(!doFit)  return(rho)
 
   ## Fit the clmm:
   fit <-
-    if(ntau == 1L) clmm.fit.ssr(rho, control = control$optCtrl, Hess)
-    else  clmm.fit.env(rho, control = control$optCtrl, Hess)
+    if(use.ssr) clmm.fit.ssr(rho, control = control$optCtrl, Hess)
+    else clmm.fit.env(rho, control = control$optCtrl, Hess)
   
   ## Modify and return results:
   fit$nAGQ <- nAGQ
@@ -93,10 +100,13 @@ more than one random effects term", call.=FALSE)
   fit$start <- start
   fit$threshold <- threshold
   fit$call <- match.call()
+  fit$formula <- frames$formula
   fit$tJac <- ths$tJac
   fit$contrasts <- attr(frames$X, "contrasts")
   fit$na.action <- attr(frames$mf, "na.action")
   fit$terms <- frames$terms
+### FIXME: Should the terms object contain only the fixed effects
+### terms? 
   fit$xlevels <- .getXlevels(fit$terms, frames$mf)
   fit$y.levels <- levels(frames$y)
   res <- clmm.finalize(fit=fit, frames=frames,
@@ -112,20 +122,29 @@ clmm.model.frame <- function(mc, contrasts) {
 ### mc - the matched call
 ### contrasts - contrasts for the fixed model terms
 
-### FIXME: what if formula is not "evaluated", but just
-### frm <- y ~ x; clmm2(frm, ...)?
+  ## Evaluate the formula in the enviroment in which clmm was called
+  ## (parent.frame(2)) to get them evaluated properly:
+  form <- eval.parent(mc$formula, 2)
+  ## get the environment of the formula. If this does not have an
+  ## enviroment (it could be a character), then use the parent frame. 
+  form.envir <-
+    if(!is.null(env <- environment(form))) env
+    else parent.frame(2)
+  ## ensure 'formula' is a formula-object:
+  form <- try(formula(deparse(form), env = form.envir), silent=TRUE)
+  ## report error if the formula cannot be interpreted
+  if(class(form) == "try-error")
+    stop("unable to interpret 'formula'")
+  environment(form) <- form.envir
 
   ## Construct a formula with all (fixed and random) variables
   ## (fullForm) and a formula with only fixed-effects variables
   ## (fixedForm):  
-  fixedForm <- nobars(mc$formula) ## ignore terms with '|'
-  fullForm <- subbars(mc$formula)      # substitute `+' for `|'
+  fixedForm <- nobars(form) ## ignore terms with '|'
+  fullForm <- subbars(form)      # substitute `+' for `|'
   
   ## Set the appropriate environments:
-  environment(fullForm) <- environment(fixedForm) <-
-    environment(eval(mc$formula))
-### FIXME: possibly environment(eval.parent(mc$formula, 2)) is
-### better?? 
+  environment(fullForm) <- environment(fixedForm) <- form.envir
 
   ## Extract full model.frame (fullmf):
   m <- match(c("data", "subset", "weights", "na.action", "offset"),
@@ -149,6 +168,7 @@ clmm.model.frame <- function(mc, contrasts) {
   fixedmf <- eval(fixedmf, envir = parent.frame(2))
   fixedTerms <- attr(fixedmf, "terms")
   X <- model.matrix(fixedTerms, fullmf, contrasts)
+  n <- nrow(X)
   ## remove intercept from X:
   Xint <- match("(Intercept)", colnames(X), nomatch = 0)
   if(Xint <= 0) {
@@ -157,7 +177,7 @@ clmm.model.frame <- function(mc, contrasts) {
   } ## intercept in X is garanteed.
 
   ## Make grList:
-  barList <- expandSlash(findbars(mc$formula[[3]]))
+  barList <- expandSlash(findbars(form[[3]]))
   if (!length(barList))
     stop("No random effects terms specified in formula")
   names(barList) <- unlist(lapply(barList,
@@ -202,7 +222,7 @@ clmm.model.frame <- function(mc, contrasts) {
   attr(fullmf, "terms") <- fixedTerms
   res <- list(y = y, X = X, Zt = Zt, wts = as.double(wts),
               off = as.double(off), mf = fullmf, terms = fixedTerms,
-              grList = grList)
+              grList = grList, formula = form)
   ## Note: X is with dimnames and an intercept is guarantied.
   return(res)
 }
@@ -274,16 +294,22 @@ clmm.fit.env <-
 
 getNLA <- function(rho, par) {
 ### negative log-likelihood by the Laplace approximation
-    if(!missing(par)) rho$par <- par
-    if(!update.u(rho)) return(Inf)
-    if(any(rho$D < 0)) return(Inf)
-    logDetD <- c(suppressWarnings(determinant(rho$L)$modulus)) -
-      rho$nrandom * log(2*pi) / 2
-    rho$nll + logDetD 
+  if(!missing(par)) rho$par <- par
+  if(any(!is.finite(rho$par)))
+    stop(gettextf(paste(c("Non-finite parameters not allowed:",
+                          formatC(rho$par, format="g")), collapse=" ")))
+  if(!update.u(rho)) return(Inf)
+  if(any(rho$D < 0)) return(Inf)
+  logDetD <- c(suppressWarnings(determinant(rho$L)$modulus)) -
+    rho$nrandom * log(2*pi) / 2
+  rho$nll + logDetD
 }
 
 nll.u <- function(rho) { ## negative log-likelihood
 ### Not allowing for scale, and flexible link functions
+  ## if(any(diff(rho$par[1:rho$nalpha]) < 0)) 
+  ##   return(Inf)
+### FIXME: adjust for threshold functions!
   rho$tau <- with(rho, exp(par[nalpha + nbeta + 1:ntau]))
   rho$varVec <- rep.int(rho$tau, rho$nlev)
   ## rho$b <- rho$varVec * rho$u
@@ -293,8 +319,8 @@ nll.u <- function(rho) { ## negative log-likelihood
 ### FIXME: possibly it should be '- b.exploded' ?
   rho$eta1 <- as.vector(rho$eta1Fix + b.exploded + rho$o1)
   rho$eta2 <- as.vector(rho$eta2Fix + b.exploded + rho$o2)
-  rho$fitted <- rho$pfun(rho$eta1) - rho$pfun(rho$eta2)
-  if(any(is.na(rho$fitted)) || any(rho$fitted <= 0))
+  rho$fitted <- getFittedC(rho$eta1, rho$eta2, rho$link)
+  if(any(!is.finite(rho$fitted)) || any(rho$fitted <= 0))
     nll <- Inf
   else
     nll <- -sum(rho$wts * log(rho$fitted)) -
@@ -302,16 +328,19 @@ nll.u <- function(rho) { ## negative log-likelihood
   nll
 }
 
-
 nllFast.u <- function(rho) { ## negative log-likelihood
 ### Not allowing for scale, and flexible link functions
+  ## If the thresholds are not increasing, return Inf:
+  ## if(any(diff(rho$par[1:rho$nalpha]) < 0))
+  ##   return(Inf)
+### FIXME: adjust for threshold functions!
   rho$varVec <- rep.int(rho$tau, rho$nlev)
   b.exploded <- as.vector(crossprod(rho$Zt, rho$varVec * rho$u))
 ### FIXME: possibly it should be '- b.exploded' ?
   rho$eta1 <- as.vector(rho$eta1Fix + b.exploded + rho$o1)
   rho$eta2 <- as.vector(rho$eta2Fix + b.exploded + rho$o2)
-  rho$fitted <- rho$pfun(rho$eta1) - rho$pfun(rho$eta2)
-  if(any(is.na(rho$fitted)) || any(rho$fitted <= 0))
+  rho$fitted <- getFittedC(rho$eta1, rho$eta2, rho$link)
+  if(any(!is.finite(rho$fitted)) || any(rho$fitted <= 0))
     nll <- Inf
   else
     nll <- -sum(rho$wts * log(rho$fitted)) -
@@ -319,7 +348,6 @@ nllFast.u <- function(rho) { ## negative log-likelihood
   nll
 }
 
-
 grad.u <- function(rho){
 ### should only be called with up to date values of eta1, eta2, par
   ## compute phi1:
@@ -333,16 +361,22 @@ grad.u <- function(rho){
 hess.u <- function(rho) {
 ### should only be called with up-to-date values of eta1, eta2, par,
 ### p1, p2
-  ## browser()
   g1 <- rho$gfun(rho$eta1) ## does not need to be saved
   g2 <- rho$gfun(rho$eta2) ## does not need to be saved
-  phi2 <- ((rho$p1 - rho$p2)^2 / rho$fitted - g1 + g2) * rho$wtpr
+  ## phi2 <- ((rho$p1 - rho$p2)^2 / rho$fitted - g1 + g2) * rho$wtpr
+  phi2 <- rho$wts * ( ((rho$p1 - rho$p2) / rho$fitted)^2 -
+                     ( (g1 - g2) / rho$fitted) )
   ## rho$Lambda <- Diagonal(x = rho$varVec)
   ## rho$Vt <- crossprod(rho$Lambda,
   ##                     tcrossprod(rho$Zt, Diagonal(x = sqrt(phi2))))
+  ## This may happen if the link function [pfun, dfun and gfun]
+  ## evalueates its arguments inaccurately: 
+  if(any(phi2 < 0))  return(FALSE)
   rho$Vt <- crossprod(Diagonal(x = rho$varVec),
                       tcrossprod(rho$Zt, Diagonal(x = sqrt(phi2))))
-  return(update(rho$L, rho$Vt, mult = 1))
+  rho$L <- update(rho$L, rho$Vt, mult = 1)
+  return(TRUE)
+  ## return(update(rho$L, rho$Vt, mult = 1))
   ## return(tcrossprod(rho$Vt) + Diagonal(rho$nrandom)) ## Hessian
 }
 
@@ -370,7 +404,7 @@ iterate is probably solution"
             conv <- 0
       break
     }
-    rho$L <- hess.u(rho)
+    if(!hess.u(rho)) return(FALSE)
     step <- as.vector(solve(rho$L, rho$gradient))
     rho$u <- rho$u - stepFactor * step
     nllTry <- nllFast.u(rho) ## no 'X %*% beta' update
@@ -407,7 +441,7 @@ the random effects"
   else if(conv != 0 && rho$ctrl$innerCtrl == "giveError")
         stop(message, "\n  at iteration ", rho$Niter)
   rho$Niter <- rho$Niter + i - 1
-    rho$L <- hess.u(rho)
+  if(!hess.u(rho)) return(FALSE)
   if(!is.finite(rho$nll))
     return(FALSE)
   else
@@ -476,11 +510,13 @@ clmm.finalize <-
 clmm.control <-
   function(method = c("ucminf", "model.frame"),
            ..., trace = 0, maxIter = 50, gradTol = 1e-4,
-           maxLineIter = 50,
+           maxLineIter = 50, useMatrix = FALSE,
            innerCtrl = c("warnOnly", "noWarn", "giveError"))
 {
   method <- match.arg(method)
   innerCtrl <- match.arg(innerCtrl)
+  useMatrix <- as.logical(useMatrix)
+  stopifnot(is.logical(useMatrix))
   ctrl <- list(trace=ifelse(trace < 0, 1, 0),
                maxIter=maxIter,
                gradTol=gradTol,
@@ -497,6 +533,55 @@ clmm.control <-
   if(method == "ucminf" && !"grad" %in% names(optCtrl))
     optCtrl$grad <- "central"
   
-  list(method = method, ctrl = ctrl, optCtrl = optCtrl)
+  list(method = method, useMatrix = useMatrix, ctrl = ctrl,
+       optCtrl = optCtrl)
 }
 
+## getCtrlArgs <- function(control, extras) {
+## ### Recover control arguments from clmm.control and extras (...):
+## ### 
+##   ## Collect control arguments in list:
+##   ctrl.args <- c(extras, control$method, control$useMatrix,
+##                  control$ctrl, control$optCtrl) 
+##   ## Identify the two occurences "trace", delete them, and add trace=1
+##   ## or trace=-1 to the list of arguments:
+##   which.trace <- which(names(ctrl.args) == "trace")
+##   trace.sum <- sum(unlist(ctrl.args[which.trace]))
+##   ctrl.args <- ctrl.args[-which.trace]
+##   ## remove duplicated arguments:
+##   ctrl.args <- ctrl.args[!duplicated(names(ctrl.args))]
+##   if(trace.sum >= 1) ctrl.args$trace <- 1
+##   if(trace.sum >= 2 || trace.sum <= -1) ctrl.args$trace <- -1
+##   ## return the updated list of control parameters:
+##   do.call("clmm.control", ctrl.args)
+## }
+
+getCtrlArgs <- function(control, extras) {
+### Recover control arguments from clmm.control and extras (...):
+###
+  if(!is.list(control))
+    stop("'control' should be a list")
+  ## Collect control arguments in list:
+  ## 1) assuming 'control' is a call to clmm.control:
+  ctrl.args <-
+    if(setequal(names(control),
+                c("method", "useMatrix", "ctrl", "optCtrl")))
+      c(extras, control$ctrl, control$optCtrl)
+  ## assuming 'control' is specified with control=list( 'args'):
+    else
+      c(extras, control)
+### NOTE: having c(extras, control) rather than c(control, extras)
+### means that extras have precedence over control.
+  ## Identify the two occurences "trace", delete them, and add trace=1
+  ## or trace=-1 to the list of arguments:
+  which.trace <- which(names(ctrl.args) == "trace")
+  trace.sum <- sum(unlist(ctrl.args[which.trace]))
+  if(trace.sum) 
+    ctrl.args <- ctrl.args[-which.trace]
+  ## remove duplicated arguments:
+  ctrl.args <- ctrl.args[!duplicated(names(ctrl.args))]
+  if(trace.sum >= 1) ctrl.args$trace <- 1
+  if(trace.sum >= 2 || trace.sum <= -1) ctrl.args$trace <- -1
+  ## return the updated list of control parameters:
+  do.call("clmm.control", ctrl.args)
+}
